@@ -2,11 +2,16 @@ svyhist<-function(formula, design, breaks = "Sturges",
                   include.lowest = TRUE, right = TRUE, xlab=NULL,
                   main=NULL, probability=TRUE,
                   freq=!probability,...){
+    if (inherits(design,"DBIsvydesign") || inherits(design,"ODBCsvydesign")){
+      design$variables<-getvars(formula, design$db$connection, design$db$tablename, 
+        updates = design$updates)
+      class(design)<-"survey.design2"
+    } 
     mf<-model.frame(formula,design$variables, na.action=na.pass)
     if (ncol(mf)>1) stop("Only one variable allowed.")
     variable<-mf[,1]
     varname<-names(mf)
-    h <- hist(variable,  plot=FALSE)
+    h <- hist(variable,  plot=FALSE, breaks=breaks)
     props <- coef(svymean(~cut(variable, h$breaks),
                           design, na.rm=TRUE))
     h$density<-props/diff(h$breaks)
