@@ -1,8 +1,13 @@
 ##
 ##
 twophase<-function(id,strata=NULL, probs=NULL, weights=NULL, fpc=NULL,
-                   subset, data){
-
+                   subset, data, method=c("full","approx")){
+  method<-match.arg(method)
+  if(method=="full") {
+    if (!is.null(weights)) stop("weights not accepted by method='full'")
+    twophase2(id=id, strata=strata, probs=probs, fpc=fpc,subset=subset,data=data)
+  }
+                     
   d1<-svydesign(id=id[[1]],strata=strata[[1]],weights=weights[[1]],
                 probs=probs[[1]],fpc=fpc[[1]],data=data)
 
